# Generated by Django 2.2 on 2019-05-13 15:08

from django.db import migrations


def migrate_classes_to_dedicated_model_forward(apps, schema_editor):
    Classification = apps.get_model('documents', 'Classification')
    objs = []
    ml_class_cache = {}
    for classification in Classification.objects.all().select_related('page__corpus'):
        # Cache MLClass to avoid extra requests
        try:
            ml_class = ml_class_cache[classification.page.corpus.pk, classification.class_name]
        except KeyError:
            ml_class, __ = apps.get_model('documents', 'MLClass').objects.get_or_create(
                name=classification.class_name,
                corpus=classification.page.corpus
            )
            ml_class_cache[classification.page.corpus.pk, classification.class_name] = ml_class
        classification.ml_class = ml_class
        objs.append(classification)
    Classification.objects.bulk_update(objs, ['ml_class'], batch_size=2000)


def migrate_classes_to_dedicated_model_backward(apps, schema_editor):
    Classification = apps.get_model('documents', 'Classification')
    objs = []
    for classification in Classification.objects.all().select_related('ml_class'):
        classification.class_name = classification.ml_class.name
        objs.append(classification)
    Classification.objects.bulk_update(objs, ['class_name'], batch_size=2000)


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0007_auto_20190513_1508'),
    ]

    operations = [
        migrations.RunPython(migrate_classes_to_dedicated_model_forward, migrate_classes_to_dedicated_model_backward)
    ]
