# Generated by Django 2.2 on 2019-09-27 10:34

from django.db import migrations, models
import django.db.models.deletion
import uuid


def add_default_types(apps, schema_editor):
    """
    Create types like defined in the ElementType enum (volume, page, act, surface) in each corpus.
    """
    Corpus = apps.get_model('documents', 'Corpus')
    if not Corpus.objects.exists():  # Ignore empty databases
        return

    for corpus in Corpus.objects.all():
        print('Adding default types for {}'.format(corpus.name))
        volume_type = corpus.types.create(
            slug='volume',
            display_name='Volume',
            folder=True,
        )
        page_type = corpus.types.create(
            slug='page',
            display_name='Page',
        )
        act_type = corpus.types.create(
            slug='act',
            display_name='Act',
        )
        surface_type = corpus.types.create(
            slug='surface',
            display_name='Surface',
            hidden=True,
        )

        print('Updating element types in {}'.format(corpus.name))
        corpus.elements.filter(type='volume').update(new_type=volume_type)
        corpus.elements.filter(type='page').update(new_type=page_type)
        corpus.elements.filter(type='act').update(new_type=act_type)
        corpus.elements.filter(type='surface').update(new_type=surface_type)


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0020_delete_page_act'),
    ]

    operations = [
        migrations.CreateModel(
            name='ElementNewType',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('slug', models.SlugField()),
                ('display_name', models.CharField(max_length=250)),
                ('folder', models.BooleanField(default=False)),
                ('hidden', models.BooleanField(default=False)),
                ('corpus', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='types',
                    to='documents.Corpus',
                )),
            ],
        ),
        migrations.AddField(
            model_name='element',
            name='new_type',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name='elements',
                to='documents.ElementNewType',
            ),
        ),
        migrations.AddConstraint(
            model_name='elementnewtype',
            constraint=models.UniqueConstraint(
                fields=('corpus', 'slug'),
                name='corpus_unique_type_slug',
            ),
        ),
        migrations.RunPython(
            code=add_default_types,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        ),
        migrations.AlterField(
            model_name='element',
            name='new_type',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name='elements',
                to='documents.ElementNewType',
            ),
        ),
    ]
