from django.conf import settings
from django.contrib.auth.models import BaseUserManager, Group


class UserManager(BaseUserManager):
    def create_user(self, email, password=None):
        """
        Creates and saves a User with the given email and password.
        """
        if not email:
            raise ValueError('Users must have an email address')

        user = self.model(
            email=self.normalize_email(email),
        )

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password):
        """
        Creates and saves a superuser with the given email and password.
        """
        user = self.create_user(
            email,
            password=password,
        )
        user.is_admin = True
        user.save(using=self._db)
        return user

    def create_internal_user(self, email):
        user = self.create_user(email)
        user.groups.add(Group.objects.get(id=settings.INTERNAL_GROUP_ID))
        return user
