from django.urls import reverse
from rest_framework import status
from arkindex.project.tests import FixtureTestCase
from arkindex.users.models import OAuthStatus


class TestCredentials(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.cred1 = cls.user.credentials.get()
        cls.cred2 = cls.user.credentials.create(
            provider_name='gitlab',
            provider_url='https://somewhere.else',
            token='t0k3n',
            status=OAuthStatus.Error,
        )

    def test_list_requires_login(self):
        response = self.client.get(reverse('api:credentials-list'))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_list(self):
        self.client.force_login(self.user)
        response = self.client.get(reverse('api:credentials-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertCountEqual(data['results'], [
            {
                'id': str(self.cred1.id),
                'status': 'created',
                'provider_name': 'gitlab',
                'provider_display_name': 'GitLab',
                'provider_url': 'https://somewhere',
                'account_name': None,
            },
            {
                'id': str(self.cred2.id),
                'status': 'error',
                'provider_name': 'gitlab',
                'provider_display_name': 'GitLab',
                'provider_url': 'https://somewhere.else',
                'account_name': None,
            },
        ])

    def test_list_filtered(self):
        self.client.force_login(self.user)
        response = self.client.get(reverse('api:credentials-list'), {'status': 'error'})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertCountEqual(data['results'], [
            {
                'id': str(self.cred2.id),
                'status': 'error',
                'provider_name': 'gitlab',
                'provider_display_name': 'GitLab',
                'provider_url': 'https://somewhere.else',
                'account_name': None,
            },
        ])
